#!/bin/bash
###############################################################################
# An interactive script to manage the scini module update repository.
#
# When a new repository is created, the following will occur:
# - Create a new user to be allowed for ftp login
#
# - Set up the users home directory that will become the repository
#
# - Create an rsa keys pair, and add the public key to the 
#   authorized keys list. The private key should be given to the client.
#
# - Configure ssh daemon to allow the new user and to chroot it
#   to it's home directory and nothing more.
# - Refresh SElinux policies on the new repository directory (if used)
#
# Requires
# - ssh
# - grep
# - sed
###############################################################################

#Scream if we try to substitute an undefined variable
set -u

SSHD_CONFIG_PATH=/etc/ssh/sshd_config

MENU_OPTIONS=( "Generate new RSA keys"
               "Remove the repository." 
               "Exit" )

REQ_PROGS=( "ssh" "grep" "sed" )

###############################################################################
verify_reqs() {
    local cur_prog

    for cur_prog in ${REQ_PROGS[@]}; do
        if ! which $cur_prog > /dev/null 2>&1; then
            bail_out "Required program $cur_prog was not found in \$PATH"
        fi
    done
}

###############################################################################
bail_out() {
    echo $1
    exit 1
}

###############################################################################
get_menu_sel() {
    local output_var_name=$1
    local idx
    local local_input

    echo "Plese make a selection [1 - ${#MENU_OPTIONS[@]}]: "

    while (( local_input < 1 || local_input > ${#MENU_OPTIONS[@]} )); do
        for (( idx = 0; idx < ${#MENU_OPTIONS[@]}; idx ++ )); do
            echo "$(( idx + 1 )) - ${MENU_OPTIONS[$idx]}"
        done

        read local_input
    done

    eval "$output_var_name=\$local_input"
}

###############################################################################
get_yes_no() {
    local user_input
    local ret_val=1 #NO

    while true; do
        echo -n $1
        read user_input

        user_input=$(echo $user_input | sed 'y/YESNO/yesno/')

        case $user_input in
            y|yes)
                ret_val=0
                break 
                ;;
            n|no)
                ret_val=1
                break
                ;;
            *)
                ;;
        esac
    done

    return $ret_val
}

###############################################################################

print_create_actions() {
    cat << EOF

###############################################################################
A scini module repository will be created, and the following actions 
will be taken:

- The user $user_name will be created

- A chroot directory will be created at 
    /home/$user_name/root

- A chrooted home dir will be created at
    /home/$user_name/root/home/$user_name

- A private rsa key file will be placed at 
    /home/$user_name/home/keys/$key_file

  You  should provide this key to the clients that will connect to 
  the repository

- The corresponding public key will be appended to
    /home/$user_name/.ssh/authorized_keys

  In this manner, repository clinets will connect via sftp without a password.

- /etc/ssh/sshd_config will be updated for the new user, and backup file 
  .sshd_config.bak will be created at /home/$user_name.

- sshd will be restarted.
###############################################################################
EOF
}
###############################################################################
create_rsa_key_pair() {
    local do_prompt=$1

    local ret_val=1
    local backup_tag=$(date +%s)
    local keys_backed_up=0
    local old_key

    if (( $do_prompt != 1 )) ||\
       get_yes_no "Will generate a new pair of RSA keys. Is this ok?"; then

        ssh-keygen -t rsa -P "" -N "" -f $key_file > /dev/null ||\
          bail_out "Error creating RSA keys pair."

        if [ -f /home/$user_name/keys/$key_file ]; then
            mv /home/$user_name/keys/$key_file \
               /home/$user_name/keys/backup_$key_file.$backup_tag > /dev/null 2>&1
            keys_backed_up=1
        fi
     
        if [ -f /home/$user_name/keys/$key_file.pub ]; then
            if get_yes_no "Remove old public key from authorized_keys file?"; then
                sed -i "/#REPO_KEY_BEGIN/,/#REPO_KEY_END/d" \
                       /home/$user_name/.ssh/authorized_keys
            fi

            mv /home/$user_name/keys/$key_file.pub \
               /home/$user_name/keys/backup_$key_file.pub.$backup_tag > /dev/null 2>&1

            keys_backed_up=1
        fi

        if (( $keys_backed_up == 1 )); then
            echo "Previous keys backed up with suffix $backup_tag."
        fi

        # Push public key to authorized list.
        echo "#REPO_KEY_BEGIN" >> /home/$user_name/.ssh/authorized_keys
        cat $key_file.pub >> /home/$user_name/.ssh/authorized_keys
        echo "#REPO_KEY_END" >> /home/$user_name/.ssh/authorized_keys

        mv $key_file.pub /home/$user_name/keys
        mv $key_file  /home/$user_name/keys

        ret_val=0
    fi

    return $ret_val
}

###############################################################################
delete_repository() {
    local ret_val=1
    cat << EOF
###############################################################################
You have chosen to delete the scini module repository from this machine.
The following actions will be taken:

- The user $user_name will be removed.

- The directory /home/$user_name will be RECURSIVELY DELETED.

- The section in /etc/ssh/sshd_config concerning the user $user_name will be
  removed.

- sshd will be restarted.
###############################################################################
EOF

   if get_yes_no "Is this ok?" && \
      get_yes_no "Are you sure? (Please confirm a second time)"; then

      userdel $user_name
      rm -rf /home/$user_name

      # Update /etc/ssh/sshd_config:
      # We remove lines starting from the Match $user_name line and continue
      # until the next Match block or end of file
      sed -i\
        "/[^#]*Match[ \t]*User[ \t]*$user_name/{;:loop;s/.*//g;N;/^[^#]*Match/b;\$b;b loop;};" \
        /etc/ssh/sshd_config

      service sshd restart

      echo "The scini module repository was successfully deleted."
    
      ret_val=0 #Successful deletion
   fi

   return $ret_val
}

###############################################################################
#  MAIN SCRIPT
###############################################################################
user_name=
create_repo=0
user_ok=
key_file=

user_input=

verify_reqs

# Check superuser
if [ $EUID -ne 0 ]; then
    echo "Error: Please run the script as root or with sudo."
    exit 1
fi

while [ -z "$user_name" ]; do
    echo -n "Please enter the repository username: "
    read user_input

    if echo $user_input | grep "[; \\]" > /dev/null; then
        echo "Illegal characters in username."
    else
        user_name=$(echo $user_input | tr -d " \t")
    fi
done

key_file=${user_name}_key

# Check if user exists and if so, check if it's an existing
# repository-dedicated user.

if cat /etc/passwd | grep $user_name: > /dev/null; then
    if [ ! -f /home/$user_name/.scini_repo ]; then
        echo "User $user_name already exists and is not dedicated to scini repo."
    else
        echo
        echo "User $user_name appears to be a valid scini repo user."
        echo
        user_ok=1
    fi
else
    echo "User $user_name not found."
    echo

    if get_yes_no \
        "Would you like to setup the repository with user $user_name? "; then

        print_create_actions
 
        if get_yes_no "Is this ok?"; then
            create_repo=1
        fi
    fi

    if (( $create_repo != 1 )); then
         echo "Quitting..."
    fi
fi

if (( $create_repo == 1 )); then
    # Create the repo user
    # - don't create initial homedir
    # - don't allow a shell for the user.

    useradd -M -s /bin/false $user_name > /dev/null 2>&1 ||\
      bail_out "Failed to create use $user_name"


    # Create the chroot dir (will remain owned by root)
    mkdir -p -m 755 /home/$user_name/root/

    # Create the keys dir
    mkdir -p -m 755 /home/$user_name/keys

    # Create the repo dir (will be owned by user)
    mkdir -p -m 555 /home/$user_name/root/home
    chown $user_name:$user_name /home/$user_name/root/home

    mkdir -p -m 555 /home/$user_name/root/home/$user_name
    chown $user_name:$user_name /home/$user_name/root/home/$user_name

    # Create .ssh dir to store authorized_keys file
    mkdir -p -m 555 /home/$user_name/.ssh
    chown $user_name:$user_name /home/$user_name/.ssh

    # Generate Keys (No prompt)
    create_rsa_key_pair 0

    # Create a backup of sshd_config 
    # Although it feels 'unsafe' to put it here, there is no way
    # an ftp user can read it.
    cp $SSHD_CONFIG_PATH /home/$user_name/.sshd_config.bak

    # Update sshd_config
    lines_to_add="\
Match User $user_name\n\
    X11Forwarding no\n\
    AllowTcpForwarding no\n\
    ForceCommand internal-sftp\n\
    ChrootDirectory /home/$user_name/root\n"

    if grep '^[ \t]*Subsystem[ \t][ \t]*sftp' $SSHD_CONFIG_PATH > /dev/null; then
        need_sftp_line=1
    else
        need_sftp_line=0
    fi

    if (( $need_sftp_line == 1 )); then
        #Delete previous 'Subsystem sftp' line
        sed -i '/^[ \t]*Subsystem[ \t][ \t]*sftp/d' $SSHD_CONFIG_PATH
    fi

    insert_loc=$(awk '/^[ \t]*Match/ { print NR; exit; }' $SSHD_CONFIG_PATH)

    if [ ! -z $insert_loc ]; then
        if (( $need_sftp_line == 1 )); then
            sed -i "${insert_loc}iSubsystem    sftp    internal-sftp\n" $SSHD_CONFIG_PATH
            (( insert_loc = insert_loc + 1 ))
        fi

        sed -i "${insert_loc}i${lines_to_add}" $SSHD_CONFIG_PATH
    else
        #End at the end of the file
        sed -i "\$aSubsystem    sftp    internal-sftp\n" $SSHD_CONFIG_PATH
        sed -i "\$a$lines_to_add" $SSHD_CONFIG_PATH
    fi

    # restorecon for SElinux
    if which restorecon > /dev/null 2>&1; then
      restorecon -R /home/$user_name
    fi

    # And finally restart ssh daemon
    service sshd restart

    touch /home/$user_name/.scini_repo

    echo "scini module repository was successfully created."

elif (( user_ok == 1 )); then
    # Do the menu
    while true; do
        get_menu_sel "user_input"
        case $user_input in
            1)
                if create_rsa_key_pair 1; then
                    break                 
                fi
                ;;

            2)  
                if delete_repository; then
                    break
                fi
                ;;
            3)
                echo "Quitting."
                break
                ;;
            *)
                ;;
        esac
    done
fi

exit 0
###############################################################################
# END OF MAIN SCRIPT
###############################################################################

