#!/bin/bash
###############################################################################
# This script verifies the kernel version of the ini module against the current 
# kernel (In a very rough way) and taps the scini modules repository in
# case of an inconsistency.
#
# It is expected from the ini bindir to contain "scini_cache" dir, which 
# in turn must contain conf.txt, the configuration file of this script.
###############################################################################

# Scream when substituting uninitialized vars
set -u

# Debug flag
DEBUGGING=

###############################################################################
installed_ver() {
    # Check for version file, and if doesn't exist, use hardcoded from
    # last scaleio update.
    if [ -f /opt/emc/scaleio/sdc/bin/bin/.installed ]; then
        cat /opt/emc/scaleio/sdc/bin/bin/.installed
    else
        echo 2.0.664.0
    fi
}

###############################################################################
print_sep_line()
{
  cat << EOF
###############################################################################
EOF
}

###############################################################################
goodbye() {
    echo $*
    print_sep_line
    exit 0
}

###############################################################################
bail_out() {
    echo $1
    exit 1
}

###############################################################################

ini_bindir=/bin/emc/scaleio
ini_module=scini.ko
ini_ver=$(installed_ver)

new_module=
module_file=
module_kernel=
current_kernel=
new_module_file=
cache_file=
cache_kernel=
fetch_from_repo=1

print_sep_line

echo "Verifying $ini_module against current system kernel."

# Verify scini cache dir
mkdir -p -m 700 $ini_bindir/scini_cache

# We're expecting scini.ko to be a softlink to scini_cache/...
# But if it is not, we will make it so.
if [ ! -L "$ini_bindir/$ini_module" -a -f "$ini_bindir/$ini_module" ]; then
    mkdir -p -m 700 $ini_bindir/scini_cache/unknown
    mv $ini_bindir/$ini_module $ini_bindir/scini_cache/unknown/
    ln -s -f $ini_bindir/scini_cache/unknown/$ini_module $ini_bindir/
fi

module_file=$(readlink $ini_bindir/$ini_module)
module_kernel=$(modinfo -F vermagic $module_file | sed 's/ .*$//')

current_kernel=$(uname -r)

echo "Module KRN: $module_kernel"
echo "Current KRN: $current_kernel"

[ "$current_kernel" = "$module_kernel" ] && 
  goodbye "$ini_module is up to date."

echo "Current $ini_module seems to be incompatible with your kernel."

cache_file="$ini_bindir/scini_cache/Ubuntu/$ini_ver/$current_kernel/$ini_module"

if [ -f "$cache_file" ]; then
    cache_kernel=$(modinfo -F vermagic $cache_file | sed 's/ .*$//')

    if [ "$current_kernel" = "$cache_kernel" ]; then
        echo "Found appropriate module in cache."
        new_module_file=$cache_file
        fetch_from_repo=0
    else
        echo "Error: Cached module kernel version doesn't match its path."
        echo "Will refetch from the repository."
    fi
fi

if (( $fetch_from_repo == 1 )); then
    # If we got here then we need to work the repository magic
    echo "Will attempt to fetch from module repository."

    $ini_bindir/scini_sync.sh -c $ini_bindir/scini_sync.conf                                retrieve Ubuntu/$ini_ver/$current_kernel ||        bail_out \
    "Error: Unable to fetch $ini_module. Will try to load the existing module..."

    echo "Successfully fetched $ini_module from the repository."

    # If we're here, then the download went just great.
    new_module_file=$ini_bindir/scini_cache/Ubuntu/$ini_ver/$current_kernel/$ini_module
fi

if [ ! -z "$new_module_file" ]; then
    echo "Updating soft link for $ini_module"
    ln -s -f $new_module_file $ini_bindir/
    echo "$ini_module verification/update successful."
fi

goodbye
 
